@@ -1,4 +1,5 @@
-from django.core.management.base import BaseCommand, CommandError
+from django.core.management.base import BaseCommand
+
 
 class Command(BaseCommand):
     """
@@ -8,7 +9,7 @@ class Command(BaseCommand):
     args = "[appname ...]"
     can_import_settings = True
     requires_model_validation = False
-    
+
     def handle(self, *args, **options):
         """
         Alter the tables
@@ -17,7 +18,7 @@ class Command(BaseCommand):
             from south.db import db
         except ImportError:
             raise ImproperlyConfigured("South must be installed for this command to work")
-        
+
         from categories.migration import migrate_app
         from categories import model_registry
         if args:
