@@ -1,35 +1,60 @@
 from django import template
-from django.template import Library, Node, TemplateSyntaxError, \
-    Variable, resolve_variable, VariableDoesNotExist, Context
+from django.db.models import get_model
+from django.template import (Node, TemplateSyntaxError, VariableDoesNotExist,
+                             FilterExpression)
 from categories.base import CategoryBase
 from categories.models import Category
 from mptt.utils import drilldown_tree_for_node
-from mptt.templatetags.mptt_tags import tree_path, tree_info
+from mptt.templatetags.mptt_tags import (tree_path, tree_info, RecurseTreeNode,
+                                         full_tree_for_model)
 
 register = template.Library()
 
 register.filter("category_path", tree_path)
 register.filter(tree_info)
+register.tag("full_tree_for_category", full_tree_for_model)
 
-def get_category(category_string):
+
+def resolve(var, context):
+    try:
+        return var.resolve(context)
+    except VariableDoesNotExist:
+        try:
+            return var.var
+        except AttributeError:
+            return var
+
+
+def get_cat_model(model):
+    """
+    Return a class from a string or class
+    """
+    try:
+        if isinstance(model, basestring):
+            model_class = get_model(*model.split("."))
+        elif issubclass(model, CategoryBase):
+            model_class = model
+        if model_class is None:
+            raise TypeError
+    except TypeError:
+        raise TemplateSyntaxError("Unknown model submitted: %s" % model)
+    return model_class
+
+
+def get_category(category_string, model=Category):
     """
     Convert a string, including a path, and return the Category object
     """
-    if category_string.startswith('"') and category_string.endswith('"'):
-        category = category_string[1:-1]
-    else:
-        category = category_string
-
-    if category.startswith('/'):
-        category = category[1:]
-    if category.endswith('/'):
-        category = category[:-1]
+    model_class = get_cat_model(model)
+    category = category_string.strip("'\"")
+    category = category.strip('/')
 
     cat_list = category.split('/')
     if len(cat_list) == 0:
         return None
     try:
-        categories = Category.objects.filter(name = cat_list[-1], level=len(cat_list)-1)
+        categories = model_class.objects.filter(name=cat_list[-1],
+                                          level=len(cat_list) - 1)
         if len(cat_list) == 1 and len(categories) > 1:
             return None
         # If there is only one, use it. If there is more than one, check
@@ -40,30 +65,32 @@ def get_category(category_string):
             for item in categories:
                 if item.parent.name == cat_list[-2]:
                     return item
-    except Category.DoesNotExist:
+    except model_class.DoesNotExist:
         return None
 
 
 class CategoryDrillDownNode(template.Node):
-
-    def __init__(self, category, varname):
-        self.category = template.Variable(category)
+    def __init__(self, category, varname, model):
+        self.category = category
         self.varname = varname
+        self.model = model
 
     def render(self, context):
+        category = resolve(self.category, context)
+        if isinstance(category, CategoryBase):
+            cat = category
+        else:
+            cat = get_category(category, self.model)
         try:
-            category = self.category.resolve(context)
-        except template.VariableDoesNotExist:
-            category = self.category.var
-        try:
-            if category is not None:
-                context[self.varname] = drilldown_tree_for_node(category)
+            if cat is not None:
+                context[self.varname] = drilldown_tree_for_node(cat)
             else:
                 context[self.varname] = []
-        except Category.DoesNotExist:
+        except:
             context[self.varname] = []
         return ''
 
+
 @register.tag
 def get_category_drilldown(parser, token):
     """
@@ -72,11 +99,11 @@ def get_category_drilldown(parser, token):
 
     Syntax::
 
-        {% get_category_drilldown "category name" as varname %}
+        {% get_category_drilldown "category name" [using "app.Model"] as varname %}
 
     Example::
 
-        {% get_category_drilldown "/Grandparent/Parent" as family %}
+        {% get_category_drilldown "/Grandparent/Parent" [using "app.Model"] as family %}
 
     or ::
 
@@ -86,30 +113,44 @@ def get_category_drilldown(parser, token):
 
         Grandparent, Parent, Child 1, Child 2, Child n
     """
-    bits = token.contents.split()
+    bits = token.split_contents()
     error_str = '%(tagname)s tag should be in the format {%% %(tagname)s ' \
-                '"category name" as varname %%}.'
-    if len(bits) != 4 or bits[2] != 'as':
-        raise template.TemplateSyntaxError, error_str % {'tagname': bits[0]}
-    category = bits[1]
-    varname = bits[3]
-    return CategoryDrillDownNode(category, varname)
+                '"category name" [using "app.Model"] as varname %%} or ' \
+                '{%% %(tagname)s category_obj as varname %%}.'
+    if len(bits) == 4:
+        if bits[2] != 'as':
+            raise template.TemplateSyntaxError, error_str % {'tagname': bits[0]}
+        if bits[2] == 'as':
+            varname = bits[3].strip("'\"")
+            model = "categories.category"
+    if len(bits) == 6:
+        if bits[2] not in ('using', 'as') or bits[4] not in ('using', 'as'):
+            raise template.TemplateSyntaxError, error_str % {'tagname': bits[0]}
+        if bits[2] == 'as':
+            varname = bits[3].strip("'\"")
+            model = bits[5].strip("'\"")
+        if bits[2] == 'using':
+            varname = bits[5].strip("'\"")
+            model = bits[3].strip("'\"")
+    category = FilterExpression(bits[1], parser)
+    return CategoryDrillDownNode(category, varname, model)
+
 
 @register.inclusion_tag('categories/breadcrumbs.html')
-def breadcrumbs(category,separator="/"):
+def breadcrumbs(category_string, separator=' > ', using='categories.category'):
     """
+    {% breadcrumbs category separator="::" using="categories.category" %}
+
     Render breadcrumbs, using the ``categories/breadcrumbs.html`` template,
     using the optional ``separator`` argument.
     """
-    if isinstance(category, CategoryBase):
-        cat = category
-    else:
-        cat = get_category(category)
+    cat = get_category(category_string, using)
 
     return {'category': cat, 'separator': separator}
 
+
 @register.inclusion_tag('categories/ul_tree.html')
-def display_drilldown_as_ul(category):
+def display_drilldown_as_ul(category, using='categories.Category'):
     """
     Render the category with ancestors and children using the
     ``categories/ul_tree.html`` template.
@@ -142,15 +183,12 @@ def display_drilldown_as_ul(category):
           </li>
         </ul>
     """
-    if isinstance(category, CategoryBase):
-        cat = category
-    else:
-        cat = get_category(category)
-
+    cat = get_category(category, using)
     return {'category': cat, 'path': drilldown_tree_for_node(cat) or []}
 
+
 @register.inclusion_tag('categories/ul_tree.html')
-def display_path_as_ul(category):
+def display_path_as_ul(category, using='categories.Category'):
     """
     Render the category with ancestors, but no children using the
     ``categories/ul_tree.html`` template.
@@ -180,14 +218,18 @@ def display_path_as_ul(category):
 
     return {'category': cat, 'path': cat.get_ancestors() or []}
 
+
 class TopLevelCategoriesNode(template.Node):
-    def __init__(self, varname):
+    def __init__(self, varname, model):
         self.varname = varname
+        self.model = model
 
     def render(self, context):
-        context[self.varname] = Category.objects.filter(parent=None).order_by('name')
+        model = get_cat_model(self.model)
+        context[self.varname] = model.objects.filter(parent=None).order_by('name')
         return ''
 
+
 @register.tag
 def get_top_level_categories(parser, token):
     """
@@ -195,31 +237,31 @@ def get_top_level_categories(parser, token):
 
     Syntax::
 
-        {% get_top_level_categories as categories %}
+        {% get_top_level_categories [using "app.Model"] as categories %}
 
     Returns an list of categories [<category>, <category>, <category, ...]
     """
-    bits = token.contents.split()
-    if len(bits) != 3:
-        raise template.TemplateSyntaxError(
-            "Usage: {%% %s as <variable> %%}" % bits[0]
-        )
-    if bits[1] != 'as':
-        raise template.TemplateSyntaxError(
-            "Usage: {%% %s as <variable> %%}" % bits[0]
-        )
-    return TopLevelCategoriesNode(bits[2])
+    bits = token.split_contents()
+    usage = 'Usage: {%% %s [using "app.Model"] as <variable> %%}' % bits[0]
+    if len(bits) == 3:
+        if bits[1] != 'as':
+            raise template.TemplateSyntaxError(usage)
+        varname = bits[2]
+        model = "categories.category"
+    elif len(bits) == 5:
+        if bits[1] not in ('as', 'using') and bits[3] not in ('as', 'using'):
+            raise template.TemplateSyntaxError(usage)
+        if bits[1] == 'using':
+            model = bits[2].strip("'\"")
+            varname = bits[4].strip("'\"")
+        else:
+            model = bits[4].strip("'\"")
+            varname = bits[2].strip("'\"")
 
-def resolve(var, context):
-    try:
-        return var.resolve(context)
-    except VariableDoesNotExist:
-        try:
-            return var.var
-        except AttributeError:
-            return var
+    return TopLevelCategoriesNode(varname, model)
 
-def get_latest_objects_by_category(category, app_label, model_name, set_name, 
+
+def get_latest_objects_by_category(category, app_label, model_name, set_name,
                                     date_field='pub_date', num=15):
     m = get_model(app_label, model_name)
     if not isinstance(category, CategoryBase):
@@ -229,79 +271,75 @@ def get_latest_objects_by_category(category, app_label, model_name, set_name,
     for cat in list(children) + [category]:
         if hasattr(cat, '%s_set' % set_name):
             ids.extend([x.pk for x in getattr(cat, '%s_set' % set_name).all()[:num]])
-    
+
     return m.objects.filter(pk__in=ids).order_by('-%s' % date_field)[:num]
 
+
 class LatestObjectsNode(Node):
-    def __init__(self, var_name, category, app_label, model_name, set_name, 
+    def __init__(self, var_name, category, app_label, model_name, set_name,
                  date_field='pub_date', num=15):
-        """Get latest objects of app_label.model_name"""
-        self.category = Variable(category)
-        self.app_label = Variable(app_label)
-        self.model_name = Variable(model_name)
-        self.set_name = Variable(set_name)
-        self.date_field = Variable(date_field)
-        self.num = Variable(num)
+        """
+        Get latest objects of app_label.model_name
+        """
+        self.category = category
+        self.app_label = app_label
+        self.model_name = model_name
+        self.set_name = set_name
+        self.date_field = date_field
+        self.num = num
         self.var_name = var_name
-    
-    def get_cache_key(self, category, app_label, model_name, set_name, 
-                     date_field, num):
-        """Get the cache key"""
-        key = 'latest_objects.%s' % '.'.join([category, app_label, model_name, 
-                set_name, date_field, num])
-    
+
     def render(self, context):
-        """Render this sucker"""
+        """
+        Render this sucker
+        """
         category = resolve(self.category, context)
         app_label = resolve(self.app_label, context)
         model_name = resolve(self.model_name, context)
         set_name = resolve(self.set_name, context)
         date_field = resolve(self.date_field, context)
         num = resolve(self.num, context)
-        
-        cache_key = self.get_cache_key(category, app_label, model_name, set_name, 
-                         date_field, num)
-        result = cache.get(cache_key)
-        if not result:
-            result = get_latest_objects_by_category(category, app_label, model_name, 
+
+        result = get_latest_objects_by_category(category, app_label, model_name,
                             set_name, date_field, num)
-        
-            cache.set(key, result, 300)
         context[self.var_name] = result
-        
+
         return ''
 
+
 def do_get_latest_objects_by_category(parser, token):
     """
     Get the latest objects by category
-    
+
     {% get_latest_objects_by_category category app_name model_name set_name [date_field] [number] as [var_name] %}
     """
     proper_form = "{% get_latest_objects_by_category category app_name model_name set_name [date_field] [number] as [var_name] %}"
     bits = token.split_contents()
-    
+
     if bits[-2] != 'as':
         raise TemplateSyntaxError("%s tag shoud be in the form: %s" % (bits[0], proper_form))
     if len(bits) < 7:
         raise TemplateSyntaxError("%s tag shoud be in the form: %s" % (bits[0], proper_form))
     if len(bits) > 9:
         raise TemplateSyntaxError("%s tag shoud be in the form: %s" % (bits[0], proper_form))
-    category = bits[1]
-    app_label = bits[2]
-    model_name = bits[3]
-    set_name = bits[4]
+    category = FilterExpression(bits[1], parser)
+    app_label = FilterExpression(bits[2], parser)
+    model_name = FilterExpression(bits[3], parser)
+    set_name = FilterExpression(bits[4], parser)
     var_name = bits[-1]
     if bits[5] != 'as':
-        date_field = bits[5]
+        date_field = FilterExpression(bits[5], parser)
     else:
-        date_field = None
+        date_field = FilterExpression(None, parser)
     if bits[6] != 'as':
-        num = bits[6]
+        num = FilterExpression(bits[6], parser)
     else:
-        num = None
-    return LatestObjectsNode(var_name, category, app_label, model_name, set_name, 
+        num = FilterExpression(None, parser)
+    return LatestObjectsNode(var_name, category, app_label, model_name, set_name,
                      date_field, num)
 
+register.tag("get_latest_objects_by_category", do_get_latest_objects_by_category)
+
 
 @register.filter
 def tree_queryset(value):
@@ -313,7 +351,7 @@ def tree_queryset(value):
     from copy import deepcopy
     if not isinstance(value, QuerySet):
         return value
-    
+
     qs = value
     qs2 = deepcopy(qs)
     # Reaching into the bowels of query sets to find out whether the qs is
@@ -331,9 +369,41 @@ def tree_queryset(value):
                                p.id not in include_pages:
                 ancestor_id_list = p.get_ancestors().values_list('id', flat=True)
                 include_pages.update(ancestor_id_list)
-        
+
         if include_pages:
             qs = qs | qs.model._default_manager.filter(id__in=include_pages)
-        
+
         qs = qs.distinct()
-    return qs
\ No newline at end of file
+    return qs
+
+
+@register.tag
+def recursetree(parser, token):
+    """
+    Iterates over the nodes in the tree, and renders the contained block for each node.
+    This tag will recursively render children into the template variable {{ children }}.
+    Only one database query is required (children are cached for the whole tree)
+
+    Usage:
+            <ul>
+                {% recursetree nodes %}
+                    <li>
+                        {{ node.name }}
+                        {% if not node.is_leaf_node %}
+                            <ul>
+                                {{ children }}
+                            </ul>
+                        {% endif %}
+                    </li>
+                {% endrecursetree %}
+            </ul>
+    """
+    bits = token.contents.split()
+    if len(bits) != 2:
+        raise template.TemplateSyntaxError('%s tag requires a queryset' % bits[0])
+    queryset_var = FilterExpression(bits[1], parser)
+
+    template_nodes = parser.parse(('endrecursetree',))
+    parser.delete_first_token()
+
+    return RecurseTreeNode(template_nodes, queryset_var)
