@@ -10,20 +10,21 @@ from django.utils.translation import ugettext_lazy as _
 from .models import Category
 from .settings import CACHE_VIEW_LENGTH
 
+
 @cache_page(CACHE_VIEW_LENGTH)
-def category_detail(request, path, 
+def category_detail(request, path,
     template_name='categories/category_detail.html', extra_context={}):
     path_items = path.strip('/').split('/')
     if len(path_items) >= 2:
         category = get_object_or_404(Category,
-            slug__iexact = path_items[-1],
-            level = len(path_items)-1,
+            slug__iexact=path_items[-1],
+            level=len(path_items) - 1,
             parent__slug__iexact=path_items[-2])
     else:
         category = get_object_or_404(Category,
-            slug__iexact = path_items[-1],
-            level = len(path_items)-1)
-    
+            slug__iexact=path_items[-1],
+            level=len(path_items) - 1)
+
     templates = []
     while path_items:
         templates.append('categories/%s.html' % '_'.join(path_items))
@@ -31,7 +32,7 @@ def category_detail(request, path,
     templates.append(template_name)
 
     context = RequestContext(request)
-    context.update({'category':category})
+    context.update({'category': category})
     if extra_context:
         context.update(extra_context)
     return HttpResponse(select_template(templates).render(context))
@@ -41,13 +42,13 @@ def get_category_for_path(path, queryset=Category.objects.all()):
     path_items = path.strip('/').split('/')
     if len(path_items) >= 2:
         queryset = queryset.filter(
-            slug__iexact = path_items[-1],
-            level = len(path_items)-1,
+            slug__iexact=path_items[-1],
+            level=len(path_items) - 1,
             parent__slug__iexact=path_items[-2])
     else:
         queryset = queryset.filter(
-            slug__iexact = path_items[-1],
-            level = len(path_items)-1)
+            slug__iexact=path_items[-1],
+            level=len(path_items) - 1)
     return queryset.get()
 
 try:
@@ -72,13 +73,13 @@ if ((django.VERSION[0] >= 1 and django.VERSION[1] >= 3) or HAS_CBV):
                 raise AttributeError(u"Category detail view %s must be called with "
                                      u"a %s." % self.__class__.__name__, self.path_field)
             if self.queryset is None:
-                 queryset = self.get_queryset()
+                queryset = self.get_queryset()
             try:
                 return get_category_for_path(self.kwargs[self.path_field])
             except ObjectDoesNotExist:
                 raise Http404(_(u"No %(verbose_name)s found matching the query") %
                               {'verbose_name': queryset.model._meta.verbose_name})
-        
+
         def get_template_names(self):
             names = []
             path_items = self.kwargs[self.path_field].strip('/').split('/')
@@ -88,16 +89,14 @@ if ((django.VERSION[0] >= 1 and django.VERSION[1] >= 3) or HAS_CBV):
             names.extend(super(CategoryDetailView, self).get_template_names())
             return names
 
-
     class CategoryRelatedDetail(DetailView):
-        
         path_field = 'category_path'
         object_name_field = None
-        
+
         def get_object(self, **kwargs):
             queryset = super(CategoryRelatedDetail, self).get_queryset()
             category = get_category_for_path(self.kwargs[self.path_field])
-            return queryset.get(category=category,slug=self.kwargs[self.slug_field])
+            return queryset.get(category=category, slug=self.kwargs[self.slug_field])
 
         def get_template_names(self):
             names = []
@@ -106,22 +105,22 @@ if ((django.VERSION[0] >= 1 and django.VERSION[1] >= 3) or HAS_CBV):
             if self.object_name_field:
                 path_items.append(getattr(self.object, self.object_name_field))
             while path_items:
-                names.append( '%s/category_%s_%s%s.html' % (opts.app_label,
-                                                            '_'.join(path_items),
-                                                            opts.object_name.lower(),
-                                                            self.template_name_suffix)
-                                                            )
+                names.append('%s/category_%s_%s%s.html' % (
+                    opts.app_label,
+                    '_'.join(path_items),
+                    opts.object_name.lower(),
+                    self.template_name_suffix)
+                )
                 path_items.pop()
-            names.append('%s/category_%s%s.html' % (opts.app_label, 
-                                                    opts.object_name.lower(),
-                                                    self.template_name_suffix)
-                                                    )
+            names.append('%s/category_%s%s.html' % (
+                opts.app_label,
+                opts.object_name.lower(),
+                self.template_name_suffix)
+            )
             names.extend(super(CategoryRelatedDetail, self).get_template_names())
             return names
 
-
     class CategoryRelatedList(ListView):
-        
         path_field = 'category_path'
 
         def get_queryset(self):
@@ -135,15 +134,17 @@ if ((django.VERSION[0] >= 1 and django.VERSION[1] >= 3) or HAS_CBV):
                 opts = self.object_list.model._meta
                 path_items = self.kwargs[self.path_field].strip('/').split('/')
                 while path_items:
-                    names.append( '%s/category_%s_%s%s.html' % (opts.app_label,
-                                                                '_'.join(path_items),
-                                                                opts.object_name.lower(),
-                                                                self.template_name_suffix)
-                                                                )
+                    names.append('%s/category_%s_%s%s.html' % (
+                        opts.app_label,
+                        '_'.join(path_items),
+                        opts.object_name.lower(),
+                        self.template_name_suffix)
+                    )
                     path_items.pop()
-                names.append('%s/category_%s%s.html' % (opts.app_label,
-                                                        opts.object_name.lower(),
-                                                        self.template_name_suffix)
-                                                        )
+                names.append('%s/category_%s%s.html' % (
+                    opts.app_label,
+                    opts.object_name.lower(),
+                    self.template_name_suffix)
+                )
             names.extend(super(CategoryRelatedList, self).get_template_names())
             return names
